@@ -33,7 +33,7 @@ Examples:
   square = x**2  # Square
   print square  # Prints "0.04+/-0.004"  
   print square.nominal_value  # Prints "0.04"
-  print square.std_dev()  # Prints "0.004..."
+  print square.std_dev  # Prints "0.004..."
 
   print square.derivatives[x]  # Partial derivative: 0.4 (= 2*0.20)
 
@@ -42,7 +42,7 @@ Examples:
   v = ufloat((10, 0.1), "v variable")
   sum_value = u+v
   
-  u.set_std_dev(0.1)  # Standard deviations can be updated on the fly
+  u.std_dev = 0.1  # Standard deviations can be updated on the fly
   print sum_value - u - v  # Prints "0.0" (exact result)
 
   # List of all sources of error:
@@ -209,7 +209,7 @@ is the result of x != 0.
 - This package contains tests.  They can be run either manually or
 automatically with the nose unit testing framework (nosetests).
 
-(c) 2009-2010 by Eric O. LEBIGOT (EOL) <eric.lebigot@normalesup.org>.
+(c) 2009-2013 by Eric O. LEBIGOT (EOL) <eric.lebigot@normalesup.org>.
 Please send feature requests, bug reports, or feedback to this address.
 
 Please support future development by donating $5 or more through PayPal!
@@ -232,20 +232,21 @@ import re
 import math
 from math import sqrt, log  # Optimization: no attribute look-up
 import copy
-from backport import *
 import warnings
 
+from backport import *
+
 # Numerical version:
-__version_info__ = (1, 9)
+__version_info__ = (2, 0, 1)
 __version__ = '.'.join(map(str, __version_info__))
 
-__author__ = 'Eric O. LEBIGOT (EOL)'
+__author__ = 'Eric O. LEBIGOT (EOL) <eric.lebigot@normalesup.org>'
 
 # Attributes that are always exported (some other attributes are
 # exported only if the NumPy module is available...):
 __all__ = [
 
-    # All sub-modules and packages are not imported by default,
+    # All sub-modules .and packages are not imported by default,
     # in particular because NumPy might be unavailable.
 
     'ufloat',  # Main function: returns a number with uncertainty
@@ -303,8 +304,11 @@ def deprecation(message):
     Warns the user with the given message, by issuing a
     DeprecationWarning.
     '''
-    warnings.warn(message, DeprecationWarning, stacklevel=2)
 
+    # stacklevel = 3 points to the original user call (not to the
+    # function from this module that called deprecation()).
+    # DeprecationWarning is ignored by default: not used.
+    warnings.warn(message, stacklevel=3)
 
 ###############################################################################
 
@@ -776,7 +780,7 @@ def _eq_on_aff_funcs(self, y_with_uncert):
     difference = self - y_with_uncert
     # Only an exact zero difference means that self and y are
     # equal numerically:
-    return not(difference._nominal_value or difference.std_dev())
+    return not(difference._nominal_value or difference.std_dev)
 
 def _ne_on_aff_funcs(self, y_with_uncert):
     """
@@ -820,6 +824,18 @@ def _le_on_aff_funcs(self, y_with_uncert):
 
 ########################################
 
+class CallableStdDev(float):
+    '''
+    Class for standard deviation results, which used to be
+    callable. Provided for compatibility with old code. Issues an
+    obsolescence warning upon call.
+    '''
+    
+    def __call__ (self):
+        deprecation('Obsolete: The std_dev attribute should not be called'
+                    ' anymore: use .std_dev instead of .std_dev().')
+        return self
+        
 class AffineScalarFunc(object):
     """
     Affine functions that support basic mathematical operations
@@ -980,7 +996,8 @@ class AffineScalarFunc(object):
         """
         Individual components of the standard deviation of the affine
         function (in absolute value), returned as a dictionary with
-        Variable objects as keys.
+        Variable objects as keys. The returned variables are the
+        independent variables that the affine function depends on.
 
         This method assumes that the derivatives contained in the
         object take scalar values (and are not a tuple, like what
@@ -994,7 +1011,8 @@ class AffineScalarFunc(object):
             error_components[variable] = abs(derivative*variable._std_dev)
 
         return error_components
-
+    
+    @property
     def std_dev(self):
         """
         Standard deviation of the affine function.
@@ -1002,7 +1020,7 @@ class AffineScalarFunc(object):
         This method assumes that the function returns scalar results.
 
         This returned standard deviation depends on the current
-        standard deviations [std_dev()] of the variables (Variable
+        standard deviations [std_dev] of the variables (Variable
         objects) involved.
         """
         #! It would be possible to not allow the user to update the
@@ -1011,8 +1029,8 @@ class AffineScalarFunc(object):
         #std_dev value (in fact, many intermediate AffineScalarFunc do
         #not need to have their std_dev calculated: only the final
         #AffineScalarFunc returned to the user does).
-        return sqrt(sum([
-            delta**2 for delta in self.error_components().itervalues()]))
+        return CallableStdDev(sqrt(sum([
+            delta**2 for delta in self.error_components().itervalues()])))
 
     def _general_representation(self, to_string):
         """
@@ -1023,7 +1041,7 @@ class AffineScalarFunc(object):
         to_string() is typically repr() or str().
         """
 
-        (nominal_value, std_dev) = (self._nominal_value, self.std_dev())
+        (nominal_value, std_dev) = (self._nominal_value, self.std_dev)
 
         # String representation:
 
@@ -1045,14 +1063,6 @@ class AffineScalarFunc(object):
                     
     def __str__(self):
         return self._general_representation(str)
-
-    def position_in_sigmas(self, value):
-        '''
-        Wrapper for legacy code.  Obsolete: do not use.  Use std_score
-        instead.
-        '''
-        deprecation("position_in_sigmas is obsolete.  Use std_score instead")
-        return self.std_score(value)
     
     def std_score(self, value):
         """
@@ -1064,7 +1074,7 @@ class AffineScalarFunc(object):
         try:
             # The ._nominal_value is a float: there is no integer division,
             # here:
-            return (value - self._nominal_value) / self.std_dev()
+            return (value - self._nominal_value) / self.std_dev
         except ZeroDivisionError:
             raise ValueError("The standard deviation is zero:"
                              " undefined result.")
@@ -1086,12 +1096,69 @@ class AffineScalarFunc(object):
     def __getstate__(self):
         """
         Hook for the pickle module.
+
+        The slot attributes of the parent classes are returned, as
+        well as those of the __dict__ attribute of the object (if
+        any).
         """
-        obj_slot_values = dict([(k, getattr(self, k)) for k in
-                                # self.__slots__ would not work when
-                                # self is an instance of a subclass:
-                                AffineScalarFunc.__slots__])
-        return obj_slot_values
+
+        # In general (case where this class is subclassed), data
+        # attribute are stored in two places: possibly in __dict_, and
+        # in slots. Data from both locations is returned by this
+        # method.
+        
+        all_attrs = {}
+
+        # Support for subclasses that do not use __slots__ (except
+        # through inheritance): instances have a __dict__
+        # attribute. The keys in this __dict__ are shadowed by the
+        # slot attribute names (reference:
+        # http://stackoverflow.com/questions/15139067/attribute-access-in-python-first-slots-then-dict/15139208#15139208).
+        # The method below not only preserves this behavior, but also
+        # saves the full contents of __dict__. This is robust:
+        # unpickling gives back the original __dict__ even if __dict__
+        # contains keys that are shadowed by slot names:
+
+        try:
+            all_attrs['__dict__'] = self.__dict__
+        except AttributeError:
+            pass
+        
+        # All the slot attributes are gathered.
+
+        # Classes that do not define __slots__ have the __slots__ of
+        # one of their parents (the first parent with their own
+        # __slots__ in MRO). This is why the slot names are first
+        # gathered (with repetitions removed, in general), and their
+        # values obtained later.
+        
+        all_slots = set()
+        
+        for cls in type(self).mro():
+
+            # In the diamond inheritance pattern, some parent classes
+            # may not have __slots__:
+            slot_names = getattr(cls, '__slots__', ())
+
+            # Slot names can be given in various forms (string,
+            # sequence, iterable):
+            if isinstance(slot_names, str):
+                all_slots.add(slot_names)  # Single name
+            else:
+                all_slots.update(slot_names)
+
+        # The slot values are stored:
+        for name in all_slots:
+            try:
+                # !! It might happen that '__dict__' is itself a slot
+                # name. In this case, its value is saved
+                # again. Alternatively, the loop could be done on
+                # all_slots - set(('__dict__',)):
+                all_attrs[name] = getattr(self, name)
+            except AttributeError:
+                pass  # Undefined slot attribute
+                
+        return all_attrs
 
     def __setstate__(self, data_dict):
         """
@@ -1100,7 +1167,9 @@ class AffineScalarFunc(object):
         for (name, value) in data_dict.iteritems():
             setattr(self, name, value)
 
-# Nicer name, for users: isinstance(ufloat(...), UFloat) is True:
+# Nicer name, for users: isinstance(ufloat(...), UFloat) is
+# True. Also: isinstance(..., UFloat) is the test for "is this a
+# number with uncertainties from the uncertainties package?".
 UFloat = AffineScalarFunc
 
 def get_ops_with_reflection():
@@ -1282,39 +1351,38 @@ class Variable(AffineScalarFunc):
         # takes much more memory.  Thus, this implementation chooses
         # more cycles and a smaller memory footprint instead of no
         # cycles and a larger memory footprint.
-
-        # ! Using AffineScalarFunc instead of super() results only in
-        # a 3 % speed loss (Python 2.6, Mac OS X):
         super(Variable, self).__init__(value, {self: 1.})
 
-        # We force the error to be float-like.  Since it is considered
-        # as a Gaussian standard deviation, it is semantically
-        # positive (even though there would be no problem defining it
-        # as a sigma, where sigma can be negative and still define a
-        # Gaussian):
-
-        assert std_dev >= 0, "the error must be a positive number"
-        # Since AffineScalarFunc.std_dev is a property, we cannot do
-        # "self.std_dev = ...":
-        self._std_dev = std_dev
+        self.std_dev = std_dev  # Assignment through a Python property
         
         self.tag = tag
 
+    # !! In Python 2.6+, the std_dev property would be more simply
+    # implemented with @property(getter), then @std_dev.setter(setter).
+
     # Standard deviations can be modified (this is a feature).
     # AffineScalarFunc objects that depend on the Variable have their
-    # std_dev() automatically modified (recalculated with the new
+    # std_dev automatically modified (recalculated with the new
     # std_dev of their Variables):
-    def set_std_dev(self, value):
-        """
-        Updates the standard deviation of the variable to a new value.
-        """
+    def _set_std_dev(self, std_dev):
+    
+        # We force the error to be float-like.  Since it is considered
+        # as a standard deviation, it must be positive:
+        assert std_dev >= 0, "the error must be a positive number"
+
+        self._std_dev = CallableStdDev(std_dev)
+    
+    def _get_std_dev(self):
+        return self._std_dev
+        
+    std_dev = property(_get_std_dev, _set_std_dev)
+    
+    # Support for legacy method:
+    def set_std_dev(self, value):  # Obsolete
+        deprecation('Obsolete: instead of set_std_dev(), please use'
+                    ' .std_dev = ...')
+        self.std_dev = value
         
-        # A zero variance is accepted.  Thus, it is possible to
-        # conveniently use infinitely precise variables, for instance
-        # to study special cases.
-
-        self._std_dev = value
-
     # The following method is overridden so that we can represent the tag:
     def _general_representation(self, to_string):
         """
@@ -1357,7 +1425,7 @@ class Variable(AffineScalarFunc):
         #! The following assumes that the arguments to Variable are
         # *not* copied upon construction, since __copy__ is not supposed
         # to copy "inside" information:
-        return Variable(self.nominal_value, self.std_dev(), self.tag)
+        return Variable(self.nominal_value, self.std_dev, self.tag)
 
     def __deepcopy__(self, memo):
         """
@@ -1374,21 +1442,6 @@ class Variable(AffineScalarFunc):
         
         return self.__copy__()
 
-    def __getstate__(self):
-        """
-        Hook for the standard pickle module.
-        """
-        obj_slot_values = dict([(k, getattr(self, k)) for k in self.__slots__])
-        obj_slot_values.update(AffineScalarFunc.__getstate__(self))
-        # Conversion to a usual dictionary:
-        return obj_slot_values
-
-    def __setstate__(self, data_dict):
-        """
-        Hook for the standard pickle module.
-        """        
-        for (name, value) in data_dict.iteritems():
-            setattr(self, name, value)
         
 ###############################################################################
 
@@ -1420,7 +1473,7 @@ def std_dev(x):
     """
 
     if isinstance(x, AffineScalarFunc):
-        return x.std_dev()
+        return x.std_dev
     else:
         return 0.
 
@@ -1444,11 +1497,13 @@ def covariance_matrix(nums_with_uncert):
     # See PSI.411 in EOL's notes.
 
     covariance_matrix = []
+    # ! In Python 2.6+, this could be replaced by enumerate(..., 1),
+    # along with updating places where i1 is used (i1+1 => i1).
     for (i1, expr1) in enumerate(nums_with_uncert):
         derivatives1 = expr1.derivatives  # Optimization
         vars1 = set(derivatives1)
         coefs_expr1 = []
-        for (i2, expr2) in enumerate(nums_with_uncert[:i1+1]):
+        for expr2 in nums_with_uncert[:i1+1]:
             derivatives2 = expr2.derivatives  # Optimization
             coef = 0.
             for var in vars1.intersection(derivatives2):
@@ -1558,7 +1613,7 @@ def parse_error_in_parentheses(representation):
     
 # The following function is not exposed because it can in effect be
 # obtained by doing x = ufloat(representation) and
-# x.nominal_value and x.std_dev():
+# x.nominal_value and x.std_dev:
 def str_to_number_with_uncert(representation):
     """
     Given a string that represents a number with uncertainty, returns the
