@@ -5,7 +5,7 @@ Tests of the code in uncertainties/__init__.py.
 
 These tests can be run through the Nose testing framework.
 
-(c) 2010 by Eric O. LEBIGOT (EOL).
+(c) 2010-2013 by Eric O. LEBIGOT (EOL).
 """
 
 from __future__ import division
@@ -24,10 +24,13 @@ import sys
 
 import uncertainties
 from uncertainties import ufloat, AffineScalarFunc
-
+from backport import *
 from uncertainties import __author__
 
-from backport import *
+# The following information is useful for making sure that the right
+# version of Python is running the tests (for instance with the Travis
+# Continuous Integration system):
+print "Testing with Python", sys.version
 
 ###############################################################################
 
@@ -280,7 +283,22 @@ def test_copy():
     gc.collect()
 
     assert y in y.derivatives.keys()
+
+## Classes for the pickling tests (put at the module level, so that
+## they can be unpickled):
     
+# Subclass without slots:
+class NewVariable_dict(uncertainties.Variable):
+    pass
+
+# Subclass with slots defined by a tuple:
+class NewVariable_slots_tuple(uncertainties.Variable):
+    __slots__ = ('new_attr',)
+
+# Subclass with slots defined by a string:
+class NewVariable_slots_str(uncertainties.Variable):
+    __slots__ = 'new_attr'
+        
 def test_pickling():
     "Standard pickle module integration."
 
@@ -299,7 +317,46 @@ def test_pickling():
     # Correlations must be preserved:
     assert f_unpickled - x_unpickled2 - x_unpickled2 == 0
     
+    ## Tests with subclasses:
+
+    for subclass in (NewVariable_dict, NewVariable_slots_tuple,
+                     NewVariable_slots_str):
+        
+        x = subclass(3, 0.14)
+
+        # Pickling test with possibly uninitialized slots:
+        pickle.loads(pickle.dumps(x))
+        
+        # Unpickling test:
+        x.new_attr = 'New attr value'
+        x_unpickled = pickle.loads(pickle.dumps(x))
+        # Must exist (From the slots of the parent class):        
+        x_unpickled.nominal_value
+        x_unpickled.new_attr  # Must exist    
+
+    ##
+        
+    # Corner case test: when an attribute is present both in __slots__
+    # and in __dict__, it is first looked up from the slots
+    # (references:
+    # http://docs.python.org/2/reference/datamodel.html#invoking-descriptors,
+    # http://stackoverflow.com/a/15139208/42973). As a consequence,
+    # the pickling process must pickle the correct value (i.e., not
+    # the value from __dict__):
+    x = NewVariable_dict(3, 0.14)
+    x._nominal_value = 'in slots'
+    # Corner case: __dict__ key which is also a slot name (it is
+    # shadowed by the corresponding slot, so this is very unusual,
+    # though):
+    x.__dict__['_nominal_value'] = 'in dict'
+    # Additional __dict__ attribute:
+    x.dict_attr = 'dict attribute'
     
+    x_unpickled = pickle.loads(pickle.dumps(x))
+    # We make sure that the data is still there and untouched:
+    assert x_unpickled._nominal_value == 'in slots'
+    assert x_unpickled.__dict__ == x.__dict__
+        
 def test_int_div():
     "Integer division"
     # We perform all operations on floats, because derivatives can
@@ -309,7 +366,7 @@ def test_int_div():
     # All errors are supposed to be small, so the ufloat()
     # in x violates the assumption.  Therefore, the following is
     # correct:
-    assert x.std_dev() == 0.0
+    assert x.std_dev == 0.0
 
 def test_comparison_ops():
     "Test of comparison operators"
@@ -390,7 +447,7 @@ def test_comparison_ops():
             infinitesimal interval withing its uncertainty.  The case
             of a zero uncertainty is special.
             """
-            return ((random.random()-0.5) * min(var.std_dev(), 1e-5)
+            return ((random.random()-0.5) * min(var.std_dev, 1e-5)
                     + var.nominal_value)
 
         # All operations are tested:
@@ -465,7 +522,11 @@ def test_logic():
     assert bool(z) == True
     assert bool(t) == True  # Only infinitseimal neighborhood are used
 
-        
+def test_obsolete():
+    'Tests obsolete functions'
+    x = ufloat((3, 0.1))
+    x.set_std_dev(0.2)  # Obsolete function
+    assert x.std_dev() == 0.2  # Obsolete use with a call
     
 def test_basic_access_to_data():
     "Access to data from Variable and AffineScalarFunc objects."
@@ -473,13 +534,13 @@ def test_basic_access_to_data():
     x = ufloat((3.14, 0.01), "x var")
     assert x.tag == "x var"
     assert x.nominal_value == 3.14
-    assert x.std_dev() == 0.01
+    assert x.std_dev == 0.01
 
     # Case of AffineScalarFunc objects:
     y = x + 0
     assert type(y) == AffineScalarFunc
     assert y.nominal_value == 3.14
-    assert y.std_dev() == 0.01
+    assert y.std_dev == 0.01
 
     # Details on the sources of error:
     a = ufloat((-1, 0.001))
@@ -493,14 +554,24 @@ def test_basic_access_to_data():
     assert y.derivatives[x] == 5
 
     # Modification of the standard deviation of variables:
-    x.set_std_dev(1)
+    x.std_dev = 1
     assert y.error_components()[x] == 5  # New error contribution!
 
+    # Calculated values with uncertainties should not have a settable
+    # standard deviation:
+    y = 2*x
+    try:
+        y.std_dev = 1
+    except AttributeError:
+        pass
+    else:
+        raise "std_dev should not be settable for calculated results"
+    
     # Calculation of deviations in units of the standard deviations:
-    assert 10/x.std_dev() == x.std_score(10 + x.nominal_value)
+    assert 10/x.std_dev == x.std_score(10 + x.nominal_value)
 
-    # "In units of the standard deviation" is not always meaningfull:
-    x.set_std_dev(0)
+    # "In units of the standard deviation" is not always meaningful:
+    x.std_dev = 0
     try:
         x.std_score(1)
     except ValueError:
@@ -513,10 +584,10 @@ def test_correlations():
     x = ufloat((4, 0.1))
     y = x*2 + a
     # Correlations cancel "naive" additions of uncertainties:
-    assert y.std_dev() != 0
+    assert y.std_dev != 0
     normally_zero = y - (x*2 + 1)
     assert normally_zero.nominal_value == 0
-    assert normally_zero.std_dev() == 0
+    assert normally_zero.std_dev == 0
 
 def test_str_input():
 
@@ -554,7 +625,7 @@ def test_str_input():
         num = ufloat(representation)
 
         assert _numbers_close(num.nominal_value, values[0])
-        assert _numbers_close(num.std_dev(), values[1])
+        assert _numbers_close(num.std_dev, values[1])
 
     
 def test_no_coercion():
@@ -607,8 +678,8 @@ def test_access_to_std_dev():
     y = 2*x
 
     # std_dev for Variable and AffineScalarFunc objects:
-    assert uncertainties.std_dev(x) == x.std_dev()
-    assert uncertainties.std_dev(y) == y.std_dev()
+    assert uncertainties.std_dev(x) == x.std_dev
+    assert uncertainties.std_dev(y) == y.std_dev
 
     # std_dev for other objects:
     assert uncertainties.std_dev([]) == 0
@@ -759,8 +830,8 @@ else:
                                   elmt2.nominal_value, precision):
                 return False
 
-            if not _numbers_close(elmt1.std_dev(),
-                                  elmt2.std_dev(), precision):
+            if not _numbers_close(elmt1.std_dev,
+                                  elmt2.std_dev, precision):
                 return False
         return True
 
@@ -831,7 +902,7 @@ else:
 
         # Test of the diagonal covariance elements:
         assert matrices_close(
-            numpy.array([v.std_dev()**2 for v in (x, y, z)]),
+            numpy.array([v.std_dev**2 for v in (x, y, z)]),
             numpy.array(covs).diagonal())
         
         # "Inversion" of the covariance matrix: creation of new
@@ -906,7 +977,7 @@ else:
         # correlation matrix (not through the covariance matrix):
 
         nominal_values = [v.nominal_value for v in (x, y, z)]
-        std_devs = [v.std_dev() for v in (x, y, z)]
+        std_devs = [v.std_dev for v in (x, y, z)]
         x2, y2, z2 = uncertainties.correlated_values_norm(
             zip(nominal_values, std_devs), corr_mat)
         
