@@ -3,7 +3,7 @@ Tests of the code in uncertainties/unumpy/__init__.py.
 
 These tests can be run through the Nose testing framework.
 
-(c) 2010 by Eric O. LEBIGOT (EOL).
+(c) 2010-2013 by Eric O. LEBIGOT (EOL).
 """
 
 from __future__ import division
@@ -148,8 +148,8 @@ def test_inverse():
     # derivatives, which define covariances:
     assert _numbers_close(m_double_inverse[0, 0].nominal_value,
                           m[0, 0].nominal_value)
-    assert _numbers_close(m_double_inverse[0, 0].std_dev(),
-                          m[0, 0].std_dev())
+    assert _numbers_close(m_double_inverse[0, 0].std_dev,
+                          m[0, 0].std_dev)
 
     assert matrices_close(m_double_inverse, m)
 
@@ -231,12 +231,12 @@ def test_array_and_matrix_creation():
     arr = unumpy.uarray(([1, 2], [0.1, 0.2]))
 
     assert arr[1].nominal_value == 2
-    assert arr[1].std_dev() == 0.2
+    assert arr[1].std_dev == 0.2
 
     # Same thing for matrices:
     mat = unumpy.umatrix(([1, 2], [0.1, 0.2]))
     assert mat[0,1].nominal_value == 2
-    assert mat[0,1].std_dev() == 0.2
+    assert mat[0,1].std_dev == 0.2
     
 def test_component_extraction():
     "Extracting the nominal values and standard deviations from an array"
