@@ -3,7 +3,7 @@ Tests of the code in uncertainties/unumpy/__init__.py.
 
 These tests can be run through the Nose testing framework.
 
-(c) 2010 by Eric O. LEBIGOT (EOL).
+(c) 2010-2013 by Eric O. LEBIGOT (EOL).
 """
 
 from __future__ import division
@@ -94,13 +94,11 @@ def _derivatives_close(x, y):
     """
 
     # x and y must depend on the same variables:
-    if len(set(x.derivatives)\
-           .symmetric_difference(y.derivatives)):
+    if set(x.derivatives) != set(y.derivatives):
         return False  # Not the same variables
 
-    return all( _numbers_close(x.derivatives[var],
-                               y.derivatives[var])
-                for var in x.derivatives)
+    return all(_numbers_close(x.derivatives[var], y.derivatives[var])
+               for var in x.derivatives)
 
 def test_inverse():
     "Tests of the matrix inverse"
@@ -148,8 +146,8 @@ def test_inverse():
     # derivatives, which define covariances:
     assert _numbers_close(m_double_inverse[0, 0].nominal_value,
                           m[0, 0].nominal_value)
-    assert _numbers_close(m_double_inverse[0, 0].std_dev(),
-                          m[0, 0].std_dev())
+    assert _numbers_close(m_double_inverse[0, 0].std_dev,
+                          m[0, 0].std_dev)
 
     assert matrices_close(m_double_inverse, m)
 
@@ -231,12 +229,12 @@ def test_array_and_matrix_creation():
     arr = unumpy.uarray(([1, 2], [0.1, 0.2]))
 
     assert arr[1].nominal_value == 2
-    assert arr[1].std_dev() == 0.2
+    assert arr[1].std_dev == 0.2
 
     # Same thing for matrices:
     mat = unumpy.umatrix(([1, 2], [0.1, 0.2]))
     assert mat[0,1].nominal_value == 2
-    assert mat[0,1].std_dev() == 0.2
+    assert mat[0,1].std_dev == 0.2
     
 def test_component_extraction():
     "Extracting the nominal values and standard deviations from an array"
@@ -255,18 +253,6 @@ def test_component_extraction():
     assert type(unumpy.nominal_values(mat)) == numpy.matrix
     
 
-def test_legacy_code():
-
-    "Test of legacy functions"
-    
-    arr_def = ([1, 2], [0.1, 0.01])
-
-    # uarray, nominal_values, std_devs from the uncertainties module
-    # (not the uncertainties.unumpy module):
-    arr = uncertainties.array_u(arr_def)  # Warning is normal
-    assert numpy.all(uncertainties.nominal_values(arr) == arr_def[0])  # Warning is normal
-    assert numpy.all(uncertainties.std_devs(arr) == arr_def[1])  # Warning is normal
-    
 def test_array_comparisons():
     "Test of array and matrix comparisons"
 
